@@ -16,13 +16,14 @@ import functools
 import io
 import logging
 import urllib.parse
+from typing import Optional
 
 import tornado.httpclient
 import tornado.websocket
 
 import intdash.data
 from intdash import _models, _protocol, _utils
-from intdash._client import USER_AGENT
+from intdash._client import USER_AGENT, TokenSource
 
 logger = logging.getLogger(__name__)
 
@@ -47,7 +48,13 @@ __all__ = [
 class ISCPConn(object):
     """WebSocket 接続を表すオブジェクトです。"""
 
-    def __init__(self, client, flush_interval, on_close=None):
+    def __init__(
+        self,
+        client,
+        flush_interval,
+        on_close=None,
+        token_source: Optional[TokenSource] = None,
+    ):
         self._logger = logger
         self._client = client
         self._flush_interval = flush_interval
@@ -78,6 +85,10 @@ class ISCPConn(object):
         self._closing = False
         self._closed = asyncio.Event()
 
+        if token_source is not None and not isinstance(token_source, TokenSource):
+            raise TypeError("Must be TokenSource class")
+        self._token_source = token_source
+
     async def __aenter__(self):
         return self
 
@@ -87,7 +98,11 @@ class ISCPConn(object):
 
     async def _start(self):
         # auth
-        self._client._auth()
+        if self._token_source is not None:
+            tk = self._token_source.token()
+        else:
+            self._client._auth()
+            tk = self._client.jwt
 
         url = urllib.parse.urlparse(self._client.url)
 
@@ -96,13 +111,13 @@ class ISCPConn(object):
         ), "shceme must be http or https"
 
         assert (
-            self._client.edge_token or self._client.jwt
+            self._client.edge_token or tk
         ), "neither edge_token nor access_token is needed"
 
         req = create_connect_request(
             url=self._client.url,
             edge_token=self._client.edge_token,
-            access_token=self._client.jwt,
+            access_token=tk,
         )
 
         # connect websocket
